@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,12 +15,14 @@
 import os
 import signal
 import sys
-from typing import Any, Dict
+from typing import Any, Dict, List, Optional
 
 import click
 import tornado.ioloop
+from streamlit import session_data
 from streamlit.git_util import GitRepo, MIN_GIT_VERSION
 
+from streamlit import version
 from streamlit import config
 from streamlit import net_util
 from streamlit import url_util
@@ -28,11 +30,11 @@ from streamlit import env_util
 from streamlit import secrets
 from streamlit import util
 from streamlit.config import CONFIG_FILENAMES
-from streamlit.report import Report
 from streamlit.logger import get_logger
 from streamlit.secrets import SECRETS_FILE_LOC
 from streamlit.server.server import Server, server_address_is_unix_socket
 from streamlit.watcher.file_watcher import watch_file
+from streamlit.watcher.file_watcher import report_watchdog_availability
 
 LOGGER = get_logger(__name__)
 
@@ -41,13 +43,28 @@ LOGGER = get_logger(__name__)
 # This must be >= 2 * WebSocketConnection.ts#RECONNECT_WAIT_TIME_MS.
 BROWSER_WAIT_TIMEOUT_SEC = 1
 
+NEW_VERSION_TEXT = """
+  %(new_version)s
 
-def _set_up_signal_handler():
+  See what's new at https://discuss.streamlit.io/c/announcements
+
+  Enter the following command to upgrade:
+  %(prompt)s %(command)s
+""" % {
+    "new_version": click.style(
+        "A new version of Streamlit is available.", fg="blue", bold=True
+    ),
+    "prompt": click.style("$", fg="blue"),
+    "command": click.style("pip install streamlit --upgrade", bold=True),
+}
+
+
+def _set_up_signal_handler() -> None:
     LOGGER.debug("Setting up signal handler")
 
     def signal_handler(signal_number, stack_frame):
         # The server will shut down its threads and stop the ioloop
-        Server.get_current().stop()
+        Server.get_current().stop(from_signal=True)
 
     signal.signal(signal.SIGTERM, signal_handler)
     signal.signal(signal.SIGINT, signal_handler)
@@ -57,7 +74,7 @@ def _set_up_signal_handler():
         signal.signal(signal.SIGQUIT, signal_handler)
 
 
-def _fix_sys_path(script_path):
+def _fix_sys_path(script_path: str) -> None:
     """Add the script's folder to the sys path.
 
     Python normally does this automatically, but since we exec the script
@@ -66,7 +83,7 @@ def _fix_sys_path(script_path):
     sys.path.insert(0, os.path.dirname(script_path))
 
 
-def _fix_matplotlib_crash():
+def _fix_matplotlib_crash() -> None:
     """Set Matplotlib backend to avoid a crash.
 
     The default Matplotlib backend crashes Python on OSX when run on a thread
@@ -94,7 +111,7 @@ def _fix_matplotlib_crash():
             pass
 
 
-def _fix_tornado_crash():
+def _fix_tornado_crash() -> None:
     """Set default asyncio policy to be compatible with Tornado 6.
 
     Tornado 6 (at least) is not compatible with the default
@@ -128,7 +145,7 @@ def _fix_tornado_crash():
                 asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())
 
 
-def _fix_sys_argv(script_path, args):
+def _fix_sys_argv(script_path: str, args: List[str]) -> None:
     """sys.argv needs to exclude streamlit arguments and parameters
     and be set to what a user's script may expect.
     """
@@ -137,9 +154,11 @@ def _fix_sys_argv(script_path, args):
     sys.argv = [script_path] + list(args)
 
 
-def _on_server_start(server):
+def _on_server_start(server: Server) -> None:
     _maybe_print_old_git_warning(server.script_path)
     _print_url(server.is_running_hello)
+    report_watchdog_availability()
+    _print_new_version_message()
 
     # Load secrets.toml if it exists. If the file doesn't exist, this
     # function will return without raising an exception. We catch any parse
@@ -170,7 +189,7 @@ def _on_server_start(server):
         else:
             addr = "localhost"
 
-        util.open_browser(Report.get_url(addr))
+        util.open_browser(session_data.get_url(addr))
 
     # Schedule the browser to open using the IO Loop on the main thread, but
     # only if no other browser connects within 1s.
@@ -178,13 +197,18 @@ def _on_server_start(server):
     ioloop.call_later(BROWSER_WAIT_TIMEOUT_SEC, maybe_open_browser)
 
 
-def _fix_pydeck_mapbox_api_warning():
+def _fix_pydeck_mapbox_api_warning() -> None:
     """Sets MAPBOX_API_KEY environment variable needed for PyDeck otherwise it will throw an exception"""
 
     os.environ["MAPBOX_API_KEY"] = config.get_option("mapbox.token")
 
 
-def _print_url(is_running_hello):
+def _print_new_version_message() -> None:
+    if version.should_show_new_version_notice():
+        click.secho(NEW_VERSION_TEXT)
+
+
+def _print_url(is_running_hello: bool) -> None:
     if is_running_hello:
         title_message = "Welcome to Streamlit. Check out our demo in your browser."
     else:
@@ -194,33 +218,33 @@ def _print_url(is_running_hello):
 
     if config.is_manually_set("browser.serverAddress"):
         named_urls = [
-            ("URL", Report.get_url(config.get_option("browser.serverAddress")))
+            ("URL", session_data.get_url(config.get_option("browser.serverAddress")))
         ]
 
     elif (
         config.is_manually_set("server.address") and not server_address_is_unix_socket()
     ):
         named_urls = [
-            ("URL", Report.get_url(config.get_option("server.address"))),
+            ("URL", session_data.get_url(config.get_option("server.address"))),
         ]
 
     elif config.get_option("server.headless"):
         internal_ip = net_util.get_internal_ip()
         if internal_ip:
-            named_urls.append(("Network URL", Report.get_url(internal_ip)))
+            named_urls.append(("Network URL", session_data.get_url(internal_ip)))
 
         external_ip = net_util.get_external_ip()
         if external_ip:
-            named_urls.append(("External URL", Report.get_url(external_ip)))
+            named_urls.append(("External URL", session_data.get_url(external_ip)))
 
     else:
         named_urls = [
-            ("Local URL", Report.get_url("localhost")),
+            ("Local URL", session_data.get_url("localhost")),
         ]
 
         internal_ip = net_util.get_internal_ip()
         if internal_ip:
-            named_urls.append(("Network URL", Report.get_url(internal_ip)))
+            named_urls.append(("Network URL", session_data.get_url(internal_ip)))
 
     click.secho("")
     click.secho("  %s" % title_message, fg="blue", bold=True)
@@ -238,6 +262,7 @@ def _print_url(is_running_hello):
         click.secho("")
         click.secho("  May you create awesome apps!")
         click.secho("")
+        click.secho("")
 
 
 def _maybe_print_old_git_warning(script_path: str) -> None:
@@ -261,13 +286,13 @@ def _maybe_print_old_git_warning(script_path: str) -> None:
             fg="yellow",
         )
         click.secho(
-            "  Git is used by Streamlit Sharing (https://streamlit.io/sharing).",
+            "  Git is used by Streamlit Cloud (https://streamlit.io/cloud).",
             fg="yellow",
         )
         click.secho("  To enable this feature, please update Git.", fg="yellow")
 
 
-def load_config_options(flag_options: Dict[str, Any]):
+def load_config_options(flag_options: Dict[str, Any]) -> None:
     """Load config options from config.toml files, then overlay the ones set by
     flag_options.
 
@@ -293,7 +318,7 @@ def load_config_options(flag_options: Dict[str, Any]):
     config.get_config_options(force_reparse=True, options_from_flags=options_from_flags)
 
 
-def _install_config_watchers(flag_options: Dict[str, Any]):
+def _install_config_watchers(flag_options: Dict[str, Any]) -> None:
     def on_config_changed(_path):
         load_config_options(flag_options)
 
@@ -302,17 +327,15 @@ def _install_config_watchers(flag_options: Dict[str, Any]):
             watch_file(filename, on_config_changed)
 
 
-def run(script_path, command_line, args, flag_options):
+def run(
+    script_path: str,
+    command_line: Optional[str],
+    args: List[str],
+    flag_options: Dict[str, Any],
+) -> None:
     """Run a script in a separate thread and start a server for the app.
 
     This starts a blocking ioloop.
-
-    Parameters
-    ----------
-    script_path : str
-    command_line : str
-    args : [str]
-    flag_options : Dict[str, Any]
     """
     _fix_sys_path(script_path)
     _fix_matplotlib_crash()
@@ -331,10 +354,6 @@ def run(script_path, command_line, args, flag_options):
     server = Server(ioloop, script_path, command_line)
     server.start(_on_server_start)
 
-    # (Must come after start(), because this starts a new thread and start()
-    # may call sys.exit() which doesn't kill other threads.
-    server.add_preheated_report_session()
-
     # Start the ioloop. This function will not return until the
     # server is shut down.
     ioloop.start()
