@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,6 @@
 from streamlit import config as _config
 
 import os
-import re
 from typing import Optional
 
 import click
@@ -96,9 +95,13 @@ def main(ctx, log_level="info"):
     """
 
     if log_level:
-        import streamlit.logger
+        from streamlit.logger import get_logger
 
-        streamlit.logger.set_log_level(log_level.upper())
+        LOGGER = get_logger(__name__)
+        LOGGER.warning(
+            "Setting the log level using the --log_level flag is unsupported."
+            "\nUse the --logger.level flag (after your streamlit command) instead."
+        )
 
 
 @main.command("help")
@@ -108,9 +111,13 @@ def help(ctx):
     # Pretend user typed 'streamlit --help' instead of 'streamlit help'.
     import sys
 
+    # We use _get_command_line_as_string to run some error checks but don't do
+    # anything with its return value.
+    _get_command_line_as_string()
+
     assert len(sys.argv) == 2  # This is always true, but let's assert anyway.
     sys.argv[1] = "--help"
-    main()
+    main(prog_name="streamlit")
 
 
 @main.command("version")
@@ -120,6 +127,10 @@ def main_version(ctx):
     # Pretend user typed 'streamlit --version' instead of 'streamlit version'
     import sys
 
+    # We use _get_command_line_as_string to run some error checks but don't do
+    # anything with its return value.
+    _get_command_line_as_string()
+
     assert len(sys.argv) == 2  # This is always true, but let's assert anyway.
     sys.argv[1] = "--version"
     main()
@@ -197,6 +208,13 @@ def _get_command_line_as_string() -> Optional[str]:
     parent = click.get_current_context().parent
     if parent is None:
         return None
+
+    if "streamlit.cli" in parent.command_path:
+        raise RuntimeError(
+            "Running streamlit via `python -m streamlit.cli <command>` is"
+            " unsupported. Please use `python -m streamlit <command>` instead."
+        )
+
     cmd_line_as_list = [parent.command_path]
     cmd_line_as_list.extend(click.get_os_args())
     return subprocess.list2cmdline(cmd_line_as_list)
@@ -230,16 +248,20 @@ def cache():
 
 @cache.command("clear")
 def cache_clear():
-    """Clear the Streamlit on-disk cache."""
+    """Clear st.cache, st.memo, and st.singleton caches."""
+    import streamlit.legacy_caching
     import streamlit.caching
 
-    result = streamlit.caching.clear_cache()
-    cache_path = streamlit.caching.get_cache_path()
+    result = streamlit.legacy_caching.clear_cache()
+    cache_path = streamlit.legacy_caching.get_cache_path()
     if result:
         print("Cleared directory %s." % cache_path)
     else:
         print("Nothing to clear at %s." % cache_path)
 
+    streamlit.caching.memo.clear()
+    streamlit.caching.singleton.clear()
+
 
 # SUBCOMMAND: config
 
@@ -277,5 +299,35 @@ def activate_reset():
     Credentials.get_current().reset()
 
 
+# SUBCOMMAND: test
+
+
+@main.group("test", hidden=True)
+def test():
+    """Internal-only commands used for testing.
+
+    These commands are not included in the output of `streamlit help`.
+    """
+    pass
+
+
+@test.command("prog_name")
+def test_prog_name():
+    """Assert that the program name is set to `streamlit test`.
+
+    This is used by our cli-smoke-tests to verify that the program name is set
+    to `streamlit ...` whether the streamlit binary is invoked directly or via
+    `python -m streamlit ...`.
+    """
+    # We use _get_command_line_as_string to run some error checks but don't do
+    # anything with its return value.
+    _get_command_line_as_string()
+
+    parent = click.get_current_context().parent
+
+    assert parent is not None
+    assert parent.command_path == "streamlit test"
+
+
 if __name__ == "__main__":
     main()
