@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ import threading
 from typing import Any, Dict, Optional, Type, Union
 
 import tornado.web
+from streamlit.script_run_context import get_script_run_ctx
 
 import streamlit.server.routes
 from streamlit import type_util
@@ -116,29 +117,14 @@ class CustomComponent:
             import pyarrow
             from streamlit.components.v1 import component_arrow
         except ImportError:
-            import sys
-
-            if sys.version_info >= (3, 9):
-                raise StreamlitAPIException(
-                    """To use Custom Components in Streamlit, you need to install
-PyArrow. Unfortunately, PyArrow does not yet support Python 3.9.
-
-You can either switch to Python 3.8 with an environment manager like PyEnv, or stay on 3.9 by
-[installing Streamlit with conda](https://discuss.streamlit.io/t/note-installation-issues-with-python-3-9-and-streamlit/6946):
-
-`conda install -c conda-forge streamlit`
-
-"""
-                )
-            else:
-                raise StreamlitAPIException(
-                    """To use Custom Components in Streamlit, you need to install
+            raise StreamlitAPIException(
+                """To use Custom Components in Streamlit, you need to install
 PyArrow. To do so locally:
 
 `pip install pyarrow`
 
-And if you're using Streamlit Sharing, add "pyarrow" to your requirements.txt."""
-                )
+And if you're using Streamlit Cloud, add "pyarrow" to your requirements.txt."""
+            )
 
         # In addition to the custom kwargs passed to the component, we also
         # send the special 'default' and 'key' params to the component
@@ -203,6 +189,7 @@ And if you're using Streamlit Sharing, add "pyarrow" to your requirements.txt.""
                 # ui_value is an object from json, an ArrowTable proto, or a bytearray
                 return ui_value
 
+            ctx = get_script_run_ctx()
             widget_value, _ = register_widget(
                 element_type="component_instance",
                 element_proto=element.component_instance,
@@ -210,6 +197,7 @@ And if you're using Streamlit Sharing, add "pyarrow" to your requirements.txt.""
                 widget_func_name=self.name,
                 deserializer=deserialize_component,
                 serializer=lambda x: x,
+                ctx=ctx,
             )
 
             if key is not None:
