@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ import os
 import secrets
 import threading
 import toml
-import urllib
 from collections import OrderedDict
 from typing import Any, Callable, cast, Dict, Optional
 
@@ -241,22 +240,6 @@ _create_option(
     type_=bool,
 )
 
-_create_option(
-    "global.sharingMode",
-    visibility="hidden",
-    description="""
-        Configure the ability to share apps to the cloud.
-
-        Should be set to one of these values:
-        - "off" : turn off sharing.
-        - "s3" : share to S3, based on the settings under the [s3] section of
-          this config file.
-        - "file" : share to a directory on the local machine. This is
-          meaningful only for debugging Streamlit itself, and shouldn't be
-          used for production.
-        """,
-    default_val="off",
-)
 
 _create_option(
     "global.showWarningOnDirectExecution",
@@ -305,14 +288,6 @@ def _global_unit_test() -> bool:
     return False
 
 
-_create_option(
-    "global.metrics",
-    description="Whether to serve prometheus metrics from /metrics.",
-    visibility="hidden",
-    default_val=False,
-    type_=bool,
-)
-
 _create_option(
     "global.suppressDeprecationWarnings",
     description="Hide deprecation warnings in the streamlit app.",
@@ -340,6 +315,19 @@ _create_option(
     type_=int,
 )
 
+_create_option(
+    "global.dataFrameSerialization",
+    description="""
+        DataFrame serialization.
+
+        Acceptable values:
+        - 'legacy': Serialize DataFrames using Streamlit's custom format. Slow
+          but battle-tested.
+        - 'arrow': Serialize DataFrames using Apache Arrow. Much faster and versatile.""",
+    default_val="arrow",
+    type_=str,
+)
+
 
 # Config Section: Logger #
 _create_section("logger", "Settings to customize Streamlit log messages.")
@@ -510,11 +498,8 @@ def _server_headless() -> bool:
     """If false, will attempt to open a browser window on start.
 
     Default: false unless (1) we are on a Linux box where DISPLAY is unset, or
-    (2) server.liveSave is set.
+    (2) we are running in the Streamlit Atom plugin.
     """
-    if get_option("server.liveSave"):
-        return True
-
     if env_util.IS_LINUX_OR_BSD and not os.getenv("DISPLAY"):
         # We're running in Linux and DISPLAY is unset
         return True
@@ -526,16 +511,6 @@ def _server_headless() -> bool:
     return False
 
 
-@_create_option("server.liveSave", type_=bool, visibility="hidden")
-def _server_live_save() -> bool:
-    """Immediately share the app in such a way that enables live
-    monitoring, and post-run analysis.
-
-    Default: false
-    """
-    return False
-
-
 @_create_option("server.runOnSave", type_=bool)
 def _server_run_on_save() -> bool:
     """Automatically rerun script when the file is modified on disk.
@@ -576,6 +551,21 @@ def _server_port() -> int:
     return 8501
 
 
+_create_option(
+    "server.scriptHealthCheckEnabled",
+    visibility="hidden",
+    description="""
+    Flag for enabling the script health check endpoint. It used for checking if
+    a script loads successfully. On success, the endpoint will return a 200
+    HTTP status code. On failure, the endpoint will return a 503 HTTP status code.
+
+    Note: This is an experimental Streamlit internal API. The API is subject
+    to change anytime so this should be used at your own risk
+    """,
+    default_val=False,
+    type_=bool,
+)
+
 _create_option(
     "server.baseUrlPath",
     description="""
@@ -622,6 +612,15 @@ def _server_max_upload_size() -> int:
     return 200
 
 
+@_create_option("server.maxMessageSize", type_=int)
+def _server_max_message_size() -> int:
+    """Max size, in megabytes, of messages that can be sent via the WebSocket connection.
+
+    Default: 200
+    """
+    return 200
+
+
 @_create_option("server.enableWebsocketCompression", type_=bool)
 def _server_enable_websocket_compression() -> bool:
     """Enables support for websocket compression.
@@ -633,7 +632,7 @@ def _server_enable_websocket_compression() -> bool:
 
 # Config Section: Browser #
 
-_create_section("browser", "Configuration of browser front-end.")
+_create_section("browser", "Configuration of non-UI browser options.")
 
 
 @_create_option("browser.serverAddress")
@@ -645,7 +644,6 @@ def _browser_server_address() -> str:
     - Set the correct URL for CORS and XSRF protection purposes.
     - Show the URL on the terminal
     - Open the browser
-    - Tell the browser where to connect to the server when in liveSave mode.
 
     Default: 'localhost'
     """
@@ -670,13 +668,28 @@ def _browser_server_port() -> int:
     - Set the correct URL for CORS and XSRF protection purposes.
     - Show the URL on the terminal
     - Open the browser
-    - Tell the browser where to connect to the server when in liveSave mode.
 
     Default: whatever value is set in server.port.
     """
     return int(get_option("server.port"))
 
 
+# Config Section: UI #
+
+_create_section("ui", "Configuration of UI elements displayed in the browser.")
+
+_create_option(
+    "ui.hideTopBar",
+    description="""
+    Flag to hide most of the UI elements found at the top of a Streamlit app.
+
+    NOTE: This does *not* hide the hamburger menu in the top-right of an app.
+    """,
+    default_val=False,
+    type_=bool,
+)
+
+
 # Config Section: Mapbox #
 
 _create_section("mapbox", "Mapbox configuration that is being used by DeckGL.")
@@ -723,82 +736,6 @@ _create_option(
     type_=bool,
 )
 
-# Config Section: S3 #
-
-_create_section("s3", 'Configuration for when global.sharingMode is set to "s3".')
-
-
-@_create_option("s3.bucket")
-def _s3_bucket() -> Optional[str]:
-    """Name of the AWS S3 bucket to save apps.
-
-    Default: (unset)
-    """
-    return None
-
-
-@_create_option("s3.url")
-def _s3_url() -> Optional[str]:
-    """URL root for external view of Streamlit apps.
-
-    Default: (unset)
-    """
-    return None
-
-
-@_create_option("s3.accessKeyId")
-def _s3_access_key_id() -> Optional[str]:
-    """Access key to write to the S3 bucket.
-
-    Leave unset if you want to use an AWS profile.
-
-    Default: (unset)
-    """
-    return None
-
-
-@_create_option("s3.secretAccessKey")
-def _s3_secret_access_key() -> Optional[str]:
-    """Secret access key to write to the S3 bucket.
-
-    Leave unset if you want to use an AWS profile.
-
-    Default: (unset)
-    """
-    return None
-
-
-_create_option(
-    "s3.keyPrefix",
-    description="""The "subdirectory" within the S3 bucket where to save
-        apps.
-
-        S3 calls paths "keys" which is why the keyPrefix is like a
-        subdirectory. Use "" to mean the root directory.
-        """,
-    default_val="",
-)
-
-_create_option(
-    "s3.region",
-    description="""AWS region where the bucket is located, e.g. "us-west-2".
-
-        Default: (unset)
-        """,
-    default_val=None,
-)
-
-_create_option(
-    "s3.profile",
-    description="""AWS credentials profile to use.
-
-        Leave unset to use your default profile.
-
-        Default: (unset)
-        """,
-    default_val=None,
-)  # If changing the default, change S3Storage.py too.
-
 
 # Config Section: Custom Theme #
 
@@ -925,9 +862,18 @@ def _set_option(key: str, value: Any, where_defined: str) -> None:
     assert (
         _config_options is not None
     ), "_config_options should always be populated here."
-    assert key in _config_options, f'Key "{key}" is not defined.'
+    if key not in _config_options:
+        # Import logger locally to prevent circular references
+        from streamlit.logger import get_logger
 
-    _config_options[key].set_value(value, where_defined)
+        LOGGER = get_logger(__name__)
+
+        LOGGER.warning(
+            f' "{key}" is not a valid config option. If you previously had this config option set, it may have been removed.'
+        )
+
+    else:
+        _config_options[key].set_value(value, where_defined)
 
 
 def _update_config_with_toml(raw_toml: str, where_defined: str) -> None:
@@ -1063,7 +1009,7 @@ def get_config_options(
             if not os.path.exists(filename):
                 continue
 
-            with open(filename, "r") as input:
+            with open(filename, "r", encoding="utf-8") as input:
                 file_contents = input.read()
 
             _update_config_with_toml(file_contents, filename)
@@ -1092,10 +1038,8 @@ def _check_conflicts() -> None:
 
     # When using the Node server, we must always connect to 8501 (this is
     # hard-coded in JS). Otherwise, the browser would decide what port to
-    # connect to based on either:
-    #   1. window.location.port, which in dev is going to be (3000)
-    #   2. the serverPort value in manifest.json, which would work, but only
-    #   exists with server.liveSave.
+    # connect to based on window.location.port, which in dev is going to
+    # be (3000)
 
     # Import logger locally to prevent circular references
     from streamlit.logger import get_logger
@@ -1111,46 +1055,6 @@ def _check_conflicts() -> None:
             "browser.serverPort does not work when global.developmentMode is " "true."
         )
 
-    # Sharing-related conflicts
-    if get_option("global.sharingMode") == "s3":
-        assert is_manually_set("s3.bucket"), (
-            'When global.sharingMode is set to "s3", ' "s3.bucket must also be set"
-        )
-        both_are_set = is_manually_set("s3.accessKeyId") and is_manually_set(
-            "s3.secretAccessKey"
-        )
-        both_are_unset = _is_unset("s3.accessKeyId") and _is_unset("s3.secretAccessKey")
-        assert both_are_set or both_are_unset, (
-            "In config.toml, s3.accessKeyId and s3.secretAccessKey must "
-            "either both be set or both be unset."
-        )
-
-        if is_manually_set("s3.url"):
-            # If s3.url is set, ensure that it's an absolute URL.
-            # An absolute URL starts with either `scheme://` or `//` --
-            # if the configured URL does not start with either prefix,
-            # prepend it with `//` to make it absolute. (If we don't do this,
-            # and the user enters something like `url=myhost.com/reports`, the
-            # browser will assume this is a relative URL, and will prepend
-            # the hostname of the Streamlit instance to the configured URL.)
-            s3_url = get_option("s3.url")
-            parsed = urllib.parse.urlparse(s3_url)
-            if parsed.netloc == "":
-                _set_option("s3.url", "//" + s3_url, get_where_defined("s3.url"))
-
-    elif get_option("global.sharingMode") == "file" and not get_option(
-        "global.developmentMode"
-    ):
-        # Warn users that "sharingMode=file" probably isn't what they meant
-        # to do.
-        LOGGER.warning(
-            "'sharingMode' is set to 'file', but Streamlit is not configured "
-            "for development. This sharingMode is used for debugging "
-            "Streamlit itself, and is not supported for other use-cases. "
-            "\n\nTo remove this warning, set the 'sharingMode' option to "
-            "another value, or remove it from your Streamlit config."
-        )
-
     # XSRF conflicts
     if get_option("server.enableXsrfProtection"):
         if not get_option("server.enableCORS") or get_option("global.developmentMode"):
@@ -1199,8 +1103,15 @@ def on_config_parsed(
         A function that the caller can use to deregister func.
     """
 
+    # We need to use the same receiver when we connect or disconnect on the
+    # Signal. If we don't do this, then the registered receiver won't be released
+    # leading to a memory leak because the Signal will keep a reference of the
+    # callable argument. When the callable argument is an object method, then
+    # the reference to that object won't be released.
+    receiver = lambda _: func_with_lock()
+
     def disconnect():
-        return _on_config_parsed.disconnect(func_with_lock)
+        return _on_config_parsed.disconnect(receiver)
 
     def func_with_lock():
         if lock:
@@ -1212,7 +1123,7 @@ def on_config_parsed(
     if force_connect or not _config_options:
         # weak=False so that we have control of when the on_config_parsed
         # callback is deregistered.
-        _on_config_parsed.connect(lambda _: func_with_lock(), weak=False)
+        _on_config_parsed.connect(receiver, weak=False)
     else:
         func_with_lock()
 
