@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,8 @@
 
 from typing import Optional, Tuple, Any, List
 
-from streamlit.report_thread import get_report_ctx
+from streamlit import util
+from streamlit.script_run_context import get_script_run_ctx
 
 
 def make_delta_path(
@@ -37,7 +38,7 @@ def get_container_cursor(
     if root_container is None:
         return None
 
-    ctx = get_report_ctx()
+    ctx = get_script_run_ctx()
 
     if ctx is None:
         return None
@@ -57,6 +58,9 @@ class Cursor:
     get_locked_cursor() on that element's respective Cursor.
     """
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     @property
     def root_container(self) -> int:
         """The top-level container this cursor lives within - either
@@ -138,7 +142,7 @@ class RunningCursor(Cursor):
             root_container=self._root_container,
             parent_path=self._parent_path,
             index=self._index,
-            **props
+            **props,
         )
 
         self._index += 1
@@ -152,7 +156,7 @@ class LockedCursor(Cursor):
         root_container: int,
         parent_path: Tuple[int, ...] = (),
         index: int = 0,
-        **props
+        **props,
     ):
         """A locked pointer to a location in the app.
 
