@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,11 +12,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import cast
+from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
+from streamlit.type_util import Key, to_key
+from textwrap import dedent
+from typing import cast, Optional
 
 import streamlit
 from streamlit.proto.Checkbox_pb2 import Checkbox as CheckboxProto
-from streamlit.state.widgets import WidgetProto, register_widget
+from streamlit.state.widgets import register_widget
+from streamlit.state.session_state import (
+    WidgetArgs,
+    WidgetCallback,
+    WidgetKwargs,
+)
 from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
 
@@ -24,14 +32,16 @@ from .utils import check_callback_rules, check_session_state_rules
 class CheckboxMixin:
     def checkbox(
         self,
-        label,
-        value=False,
-        key=None,
-        help=None,
-        on_change=None,
-        args=None,
-        kwargs=None,
-    ):
+        label: str,
+        value: bool = False,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
+    ) -> bool:
         """Display a checkbox widget.
 
         Parameters
@@ -41,8 +51,8 @@ class CheckboxMixin:
         value : bool
             Preselect the checkbox when it first renders. This will be
             cast to bool internally.
-        key : str
-            An optional string to use as the unique key for the widget.
+        key : str or int
+            An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
@@ -54,6 +64,9 @@ class CheckboxMixin:
             An optional tuple of args to pass to the callback.
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
+        disabled : bool
+            An optional boolean, which disables the checkbox if set to True.
+            The default is False. This argument can only be supplied by keyword.
 
         Returns
         -------
@@ -67,7 +80,38 @@ class CheckboxMixin:
         >>> if agree:
         ...     st.write('Great!')
 
+        .. output::
+           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.checkbox.py
+           height: 220px
+
         """
+        ctx = get_script_run_ctx()
+        return self._checkbox(
+            label=label,
+            value=value,
+            key=key,
+            help=help,
+            on_change=on_change,
+            args=args,
+            kwargs=kwargs,
+            disabled=disabled,
+            ctx=ctx,
+        )
+
+    def _checkbox(
+        self,
+        label: str,
+        value: bool = False,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
+        ctx: Optional[ScriptRunContext] = None,
+    ) -> bool:
+        key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(
             default_value=None if value is False else value, key=key
@@ -77,10 +121,11 @@ class CheckboxMixin:
         checkbox_proto.label = label
         checkbox_proto.default = bool(value)
         checkbox_proto.form_id = current_form_id(self.dg)
+        checkbox_proto.disabled = disabled
         if help is not None:
-            checkbox_proto.help = help
+            checkbox_proto.help = dedent(help)
 
-        def deserialize_checkbox(ui_value, widget_id="") -> bool:
+        def deserialize_checkbox(ui_value: Optional[bool], widget_id: str = "") -> bool:
             return bool(ui_value if ui_value is not None else value)
 
         current_value, set_frontend_value = register_widget(
@@ -92,6 +137,7 @@ class CheckboxMixin:
             kwargs=kwargs,
             deserializer=deserialize_checkbox,
             serializer=bool,
+            ctx=ctx,
         )
 
         if set_frontend_value:
@@ -99,7 +145,7 @@ class CheckboxMixin:
             checkbox_proto.set_value = True
 
         self.dg._enqueue("checkbox", checkbox_proto)
-        return current_value
+        return cast(bool, current_value)
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
