@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,37 +12,147 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import cast, List, Optional, Union
+from streamlit.type_util import Key, to_key
+from typing import cast, overload, List, Optional, Union
+from textwrap import dedent
+
+import sys
+
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
 
 import streamlit
 from streamlit import config
 from streamlit.logger import get_logger
 from streamlit.proto.FileUploader_pb2 import FileUploader as FileUploaderProto
-from streamlit.report_thread import get_report_ctx
-from streamlit.state.widgets import register_widget, NoValue
+from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
+from streamlit.state.widgets import register_widget
+from streamlit.state.session_state import (
+    WidgetArgs,
+    WidgetCallback,
+    WidgetKwargs,
+)
 from .form import current_form_id
-from ..proto.Common_pb2 import SInt64Array
+from ..proto.Common_pb2 import (
+    FileUploaderState as FileUploaderStateProto,
+    UploadedFileInfo as UploadedFileInfoProto,
+)
 from ..uploaded_file_manager import UploadedFile, UploadedFileRec
 from .utils import check_callback_rules, check_session_state_rules
 
 LOGGER = get_logger(__name__)
 
+SomeUploadedFiles = Optional[Union[UploadedFile, List[UploadedFile]]]
+
 
 class FileUploaderMixin:
+    # Multiple overloads are defined on `file_uploader()` below to represent
+    # the different return types of `file_uploader()`.
+    # These return types differ according to the value of the `accept_multiple_files` argument.
+    # There are 2 associated variables, each with 2 options.
+    # 1. The `accept_multiple_files` argument is set as `True`,
+    #    or it is set as `False` or omitted, in which case the default value `False`.
+    # 2. The `type` argument may or may not be provided as a keyword-only argument.
+    # There must be 2x2=4 overloads to cover all the possible arguments,
+    # as these overloads must be mutually exclusive for mypy.
+
+    # 1. type is given as not a keyword-only argument
+    # 2. accept_multiple_files = True
+    @overload
     def file_uploader(
         self,
-        label,
-        type=None,
-        accept_multiple_files=False,
-        key=None,
-        help=None,
-        on_change=None,
-        args=None,
-        kwargs=None,
+        label: str,
+        type: Optional[Union[str, List[str]]],
+        accept_multiple_files: Literal[True],
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,
+        disabled: bool = False,
+    ) -> Optional[List[UploadedFile]]:
+        ...
+
+    # 1. type is given as not a keyword-only argument
+    # 2. accept_multiple_files = False or omitted
+    @overload
+    def file_uploader(
+        self,
+        label: str,
+        type: Optional[Union[str, List[str]]],
+        accept_multiple_files: Literal[False] = False,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,
+        disabled: bool = False,
+    ) -> Optional[UploadedFile]:
+        ...
+
+    # The following 2 overloads represent the cases where
+    # the `type` argument is a keyword-only argument.
+    # See https://github.com/python/mypy/issues/4020#issuecomment-737600893
+    # for the related discussions and examples.
+
+    # 1. type is skipped or a keyword argument
+    # 2. accept_multiple_files = True
+    @overload
+    def file_uploader(
+        self,
+        label: str,
+        *,
+        accept_multiple_files: Literal[True],
+        type: Optional[Union[str, List[str]]] = None,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        disabled: bool = False,
+    ) -> Optional[List[UploadedFile]]:
+        ...
+
+    # 1. type is skipped or a keyword argument
+    # 2. accept_multiple_files = False or omitted
+    @overload
+    def file_uploader(
+        self,
+        label: str,
+        *,
+        accept_multiple_files: Literal[False] = False,
+        type: Optional[Union[str, List[str]]] = None,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        disabled: bool = False,
+    ) -> Optional[UploadedFile]:
+        ...
+
+    def file_uploader(
+        self,
+        label: str,
+        type: Optional[Union[str, List[str]]] = None,
+        accept_multiple_files: bool = False,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
     ):
         """Display a file uploader widget.
         By default, uploaded files are limited to 200MB. You can configure
-        this using the `server.maxUploadSize` config option.
+        this using the `server.maxUploadSize` config option. For more info
+        on how to set config options, see
+        https://docs.streamlit.io/library/advanced-features/configuration#set-configuration-options
 
         Parameters
         ----------
@@ -58,8 +168,8 @@ class FileUploaderMixin:
             in which case the return value will be a list of files.
             Default: False
 
-        key : str
-            An optional string to use as the unique key for the widget.
+        key : str or int
+            An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
@@ -77,6 +187,11 @@ class FileUploaderMixin:
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
 
+        disabled : bool
+            An optional boolean, which disables the file uploader if set to
+            True. The default is False. This argument can only be supplied by
+            keyword.
+
         Returns
         -------
         None or UploadedFile or list of UploadedFile
@@ -119,7 +234,41 @@ class FileUploaderMixin:
         ...     bytes_data = uploaded_file.read()
         ...     st.write("filename:", uploaded_file.name)
         ...     st.write(bytes_data)
+
+        .. output::
+           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.file_uploader.py
+           height: 375px
+
         """
+        ctx = get_script_run_ctx()
+        return self._file_uploader(
+            label=label,
+            type=type,
+            accept_multiple_files=accept_multiple_files,
+            key=key,
+            help=help,
+            on_change=on_change,
+            args=args,
+            kwargs=kwargs,
+            disabled=disabled,
+            ctx=ctx,
+        )
+
+    def _file_uploader(
+        self,
+        label: str,
+        type: Optional[Union[str, List[str]]] = None,
+        accept_multiple_files: bool = False,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
+        ctx: Optional[ScriptRunContext] = None,
+    ):
+        key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=None, key=key, writes_allowed=False)
 
@@ -142,12 +291,13 @@ class FileUploaderMixin:
         )
         file_uploader_proto.multiple_files = accept_multiple_files
         file_uploader_proto.form_id = current_form_id(self.dg)
+        file_uploader_proto.disabled = disabled
         if help is not None:
-            file_uploader_proto.help = help
+            file_uploader_proto.help = dedent(help)
 
         def deserialize_file_uploader(
-            ui_value: List[int], widget_id: str
-        ) -> Optional[Union[List[UploadedFile], UploadedFile]]:
+            ui_value: Optional[FileUploaderStateProto], widget_id: str
+        ) -> SomeUploadedFiles:
             file_recs = self._get_file_recs(widget_id, ui_value)
             if len(file_recs) == 0:
                 return_value: Optional[Union[List[UploadedFile], UploadedFile]] = (
@@ -158,25 +308,30 @@ class FileUploaderMixin:
                 return_value = files if accept_multiple_files else files[0]
             return return_value
 
-        def serialize_file_uploader(
-            files: Optional[Union[List[UploadedFile], UploadedFile]]
-        ) -> List[int]:
-            if not files:
-                return []
-            if isinstance(files, list):
-                ids = [f.id for f in files]
-            else:
-                ids = [files.id]
+        def serialize_file_uploader(files: SomeUploadedFiles) -> FileUploaderStateProto:
+            state_proto = FileUploaderStateProto()
 
-            ctx = get_report_ctx()
+            ctx = get_script_run_ctx()
             if ctx is None:
-                return []
+                return state_proto
 
             # ctx.uploaded_file_mgr._file_id_counter stores the id to use for
-            # the next uploaded file, so the current highest file id is the
+            # the *next* uploaded file, so the current highest file id is the
             # counter minus 1.
-            max_id = ctx.uploaded_file_mgr._file_id_counter - 1
-            return [max_id] + ids
+            state_proto.max_file_id = ctx.uploaded_file_mgr._file_id_counter - 1
+
+            if not files:
+                return state_proto
+            elif not isinstance(files, list):
+                files = [files]
+
+            for f in files:
+                file_info: UploadedFileInfoProto = state_proto.uploaded_file_info.add()
+                file_info.id = f.id
+                file_info.name = f.name
+                file_info.size = f.size
+
+            return state_proto
 
         # FileUploader's widget value is a list of file IDs
         # representing the current set of files that this uploader should
@@ -190,16 +345,14 @@ class FileUploaderMixin:
             kwargs=kwargs,
             deserializer=deserialize_file_uploader,
             serializer=serialize_file_uploader,
+            ctx=ctx,
         )
 
-        ctx = get_report_ctx()
-        if ctx is not None and widget_value:
-            serialized = serialize_file_uploader(widget_value)
-
-            # The first number in the serialized widget_value list is the id
-            # of the most recently uploaded file.
-            newest_file_id = serialized[0]
-            active_file_ids = list(serialized[1:])
+        file_uploader_state = serialize_file_uploader(widget_value)
+        uploaded_file_info = file_uploader_state.uploaded_file_info
+        if ctx is not None and len(uploaded_file_info) != 0:
+            newest_file_id = file_uploader_state.max_file_id
+            active_file_ids = [f.id for f in uploaded_file_info]
 
             ctx.uploaded_file_mgr.remove_orphaned_files(
                 session_id=ctx.session_id,
@@ -209,27 +362,24 @@ class FileUploaderMixin:
             )
 
         self.dg._enqueue("file_uploader", file_uploader_proto)
-        return widget_value
+        return cast(SomeUploadedFiles, widget_value)
 
     @staticmethod
     def _get_file_recs(
-        widget_id: str, widget_value: Optional[List[int]]
+        widget_id: str, widget_value: Optional[FileUploaderStateProto]
     ) -> List[UploadedFileRec]:
         if widget_value is None:
             return []
 
-        ctx = get_report_ctx()
+        ctx = get_script_run_ctx()
         if ctx is None:
             return []
 
-        if len(widget_value) == 0:
-            # Sanity check
-            LOGGER.warning(
-                "Got an empty FileUploader widget_value. (We expect a list with at least one value in it.)"
-            )
+        uploaded_file_info = widget_value.uploaded_file_info
+        if len(uploaded_file_info) == 0:
             return []
 
-        active_file_ids = list(widget_value[1:])
+        active_file_ids = [f.id for f in uploaded_file_info]
 
         # Grab the files that correspond to our active file IDs.
         return ctx.uploaded_file_mgr.get_files(
