@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,14 +12,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import cast, List
+from textwrap import dedent
+from typing import Any, Callable, Optional, cast, List
 
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.MultiSelect_pb2 import MultiSelect as MultiSelectProto
+from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
 from streamlit.state.widgets import register_widget
-from streamlit.type_util import OptionSequence, ensure_indexable, is_type
+from streamlit.type_util import Key, OptionSequence, ensure_indexable, is_type, to_key
 
+from streamlit.state.session_state import (
+    WidgetArgs,
+    WidgetCallback,
+    WidgetKwargs,
+)
 from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
 
@@ -27,16 +34,18 @@ from .utils import check_callback_rules, check_session_state_rules
 class MultiSelectMixin:
     def multiselect(
         self,
-        label,
+        label: str,
         options: OptionSequence,
-        default=None,
-        format_func=str,
-        key=None,
-        help=None,
-        on_change=None,
-        args=None,
-        kwargs=None,
-    ):
+        default: Optional[Any] = None,
+        format_func: Callable[[Any], Any] = str,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
+    ) -> List[Any]:
         """Display a multiselect widget.
         The multiselect widget starts as empty.
 
@@ -44,18 +53,18 @@ class MultiSelectMixin:
         ----------
         label : str
             A short label explaining to the user what this select widget is for.
-        options : Sequence, numpy.ndarray, pandas.Series, pandas.DataFrame, or pandas.Index
+        options : Sequence[V], numpy.ndarray, pandas.Series, pandas.DataFrame, or pandas.Index
             Labels for the select options. This will be cast to str internally
             by default. For pandas.DataFrame, the first column is selected.
-        default: [str] or None
-            List of default values.
+        default: [V], V, or None
+            List of default values. Can also be a single value.
         format_func : function
             Function to modify the display of selectbox options. It receives
             the raw option as an argument and should output the label to be
             shown for that option. This has no impact on the return value of
-            the selectbox.
-        key : str
-            An optional string to use as the unique key for the widget.
+            the multiselect.
+        key : str or int
+            An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
@@ -67,6 +76,10 @@ class MultiSelectMixin:
             An optional tuple of args to pass to the callback.
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
+        disabled : bool
+            An optional boolean, which disables the multiselect widget if set
+            to True. The default is False. This argument can only be supplied
+            by keyword.
 
         Returns
         -------
@@ -82,6 +95,10 @@ class MultiSelectMixin:
         >>>
         >>> st.write('You selected:', options)
 
+        .. output::
+           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.multiselect.py
+           height: 420px
+
         .. note::
            User experience can be degraded for large lists of `options` (100+), as this widget
            is not designed to handle arbitrary text search efficiently. See this
@@ -90,6 +107,37 @@ class MultiSelectMixin:
            `GitHub issue #1059 <https://github.com/streamlit/streamlit/issues/1059>`_ for updates on the issue.
 
         """
+        ctx = get_script_run_ctx()
+        return self._multiselect(
+            label=label,
+            options=options,
+            default=default,
+            format_func=format_func,
+            key=key,
+            help=help,
+            on_change=on_change,
+            args=args,
+            kwargs=kwargs,
+            disabled=disabled,
+            ctx=ctx,
+        )
+
+    def _multiselect(
+        self,
+        label: str,
+        options: OptionSequence,
+        default: Optional[Any] = None,
+        format_func: Callable[[Any], Any] = str,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
+        ctx: Optional[ScriptRunContext] = None,
+    ) -> List[Any]:
+        key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=default, key=key)
 
@@ -128,10 +176,13 @@ class MultiSelectMixin:
         multiselect_proto.default[:] = default_value
         multiselect_proto.options[:] = [str(format_func(option)) for option in opt]
         multiselect_proto.form_id = current_form_id(self.dg)
+        multiselect_proto.disabled = disabled
         if help is not None:
-            multiselect_proto.help = help
+            multiselect_proto.help = dedent(help)
 
-        def deserialize_multiselect(ui_value, widget_id="") -> List[str]:
+        def deserialize_multiselect(
+            ui_value: Optional[List[int]], widget_id: str = ""
+        ) -> List[str]:
             current_value = ui_value if ui_value is not None else default_value
             return [opt[i] for i in current_value]
 
@@ -147,6 +198,7 @@ class MultiSelectMixin:
             kwargs=kwargs,
             deserializer=deserialize_multiselect,
             serializer=serialize_multiselect,
+            ctx=ctx,
         )
 
         if set_frontend_value:
@@ -156,7 +208,7 @@ class MultiSelectMixin:
             multiselect_proto.set_value = True
 
         self.dg._enqueue("multiselect", multiselect_proto)
-        return current_value
+        return cast(List[str], current_value)
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
