@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,13 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import cast
+from textwrap import dedent
+from typing import Any, Callable, Optional, cast
 
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Radio_pb2 import Radio as RadioProto
+from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
 from streamlit.state.widgets import register_widget
-from streamlit.type_util import OptionSequence, ensure_indexable
+from streamlit.state.session_state import (
+    WidgetArgs,
+    WidgetCallback,
+    WidgetKwargs,
+)
+from streamlit.type_util import Key, OptionSequence, ensure_indexable, to_key
 from streamlit.util import index_
 from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
@@ -27,16 +34,18 @@ from .utils import check_callback_rules, check_session_state_rules
 class RadioMixin:
     def radio(
         self,
-        label,
+        label: str,
         options: OptionSequence,
-        index=0,
-        format_func=str,
-        key=None,
-        help=None,
-        on_change=None,
-        args=None,
-        kwargs=None,
-    ):
+        index: int = 0,
+        format_func: Callable[[Any], Any] = str,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only args:
+        disabled: bool = False,
+    ) -> Any:
         """Display a radio button widget.
 
         Parameters
@@ -53,8 +62,8 @@ class RadioMixin:
             the raw option as an argument and should output the label to be
             shown for that option. This has no impact on the return value of
             the radio.
-        key : str
-            An optional string to use as the unique key for the widget.
+        key : str or int
+            An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
@@ -66,6 +75,10 @@ class RadioMixin:
             An optional tuple of args to pass to the callback.
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
+        disabled : bool
+            An optional boolean, which disables the radio button if set to
+            True. The default is False. This argument can only be supplied by
+            keyword.
 
         Returns
         -------
@@ -83,7 +96,42 @@ class RadioMixin:
         ... else:
         ...     st.write("You didn\'t select comedy.")
 
+        .. output::
+           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.radio.py
+           height: 260px
+
         """
+        ctx = get_script_run_ctx()
+        return self._radio(
+            label=label,
+            options=options,
+            index=index,
+            format_func=format_func,
+            key=key,
+            help=help,
+            on_change=on_change,
+            args=args,
+            kwargs=kwargs,
+            disabled=disabled,
+            ctx=ctx,
+        )
+
+    def _radio(
+        self,
+        label: str,
+        options: OptionSequence,
+        index: int = 0,
+        format_func: Callable[[Any], Any] = str,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only args:
+        disabled: bool = False,
+        ctx: Optional[ScriptRunContext],
+    ) -> Any:
+        key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=None if index == 0 else index, key=key)
 
@@ -104,8 +152,9 @@ class RadioMixin:
         radio_proto.default = index
         radio_proto.options[:] = [str(format_func(option)) for option in opt]
         radio_proto.form_id = current_form_id(self.dg)
+        radio_proto.disabled = disabled
         if help is not None:
-            radio_proto.help = help
+            radio_proto.help = dedent(help)
 
         def deserialize_radio(ui_value, widget_id=""):
             idx = ui_value if ui_value is not None else index
@@ -126,6 +175,7 @@ class RadioMixin:
             kwargs=kwargs,
             deserializer=deserialize_radio,
             serializer=serialize_radio,
+            ctx=ctx,
         )
 
         if set_frontend_value:
@@ -133,7 +183,7 @@ class RadioMixin:
             radio_proto.set_value = True
 
         self.dg._enqueue("radio", radio_proto)
-        return current_value
+        return cast(str, current_value)
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
