@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,13 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import cast
+from textwrap import dedent
+from typing import Any, Callable, Optional, cast
 
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Slider_pb2 import Slider as SliderProto
+from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
+from streamlit.state.session_state import (
+    WidgetArgs,
+    WidgetCallback,
+    WidgetKwargs,
+)
 from streamlit.state.widgets import register_widget
-from streamlit.type_util import OptionSequence, ensure_indexable
+from streamlit.type_util import Key, OptionSequence, ensure_indexable, to_key
 from streamlit.util import index_
 from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
@@ -27,16 +34,18 @@ from .utils import check_callback_rules, check_session_state_rules
 class SelectSliderMixin:
     def select_slider(
         self,
-        label,
+        label: str,
         options: OptionSequence = [],
-        value=None,
-        format_func=str,
-        key=None,
-        help=None,
-        on_change=None,
-        args=None,
-        kwargs=None,
-    ):
+        value: Any = None,
+        format_func: Callable[[Any], Any] = str,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
+    ) -> Any:
         """
         Display a slider widget to select items from a list.
 
@@ -66,8 +75,8 @@ class SelectSliderMixin:
             Function to modify the display of the labels from the options.
             argument. It receives the option as an argument and its output
             will be cast to str.
-        key : str
-            An optional string to use as the unique key for the widget.
+        key : str or int
+            An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
@@ -79,6 +88,9 @@ class SelectSliderMixin:
             An optional tuple of args to pass to the callback.
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
+        disabled : bool
+            An optional boolean, which disables the select slider if set to True.
+            The default is False. This argument can only be supplied by keyword.
 
         Returns
         -------
@@ -100,7 +112,42 @@ class SelectSliderMixin:
         ...     options=['red', 'orange', 'yellow', 'green', 'blue', 'indigo', 'violet'],
         ...     value=('red', 'blue'))
         >>> st.write('You selected wavelengths between', start_color, 'and', end_color)
+
+        .. output::
+           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.select_slider.py
+           height: 450px
+
         """
+        ctx = get_script_run_ctx()
+        return self._select_slider(
+            label=label,
+            options=options,
+            value=value,
+            format_func=format_func,
+            key=key,
+            help=help,
+            on_change=on_change,
+            args=args,
+            kwargs=kwargs,
+            disabled=disabled,
+            ctx=ctx,
+        )
+
+    def _select_slider(
+        self,
+        label: str,
+        options: OptionSequence = [],
+        value: Any = None,
+        format_func: Callable[[Any], Any] = str,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        disabled: bool = False,
+        ctx: Optional[ScriptRunContext] = None,
+    ) -> Any:
+        key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=value, key=key)
 
@@ -110,23 +157,27 @@ class SelectSliderMixin:
             raise StreamlitAPIException("The `options` argument needs to be non-empty")
 
         is_range_value = isinstance(value, (list, tuple))
-        slider_value = value
+
+        def as_index_list(v):
+            is_range_value = isinstance(v, (list, tuple))
+            if is_range_value:
+                slider_value = [index_(opt, val) for val in v]
+                start, end = slider_value
+                if start > end:
+                    slider_value = [end, start]
+                return slider_value
+            else:
+                # Simplify future logic by always making value a list
+                try:
+                    return [index_(opt, v)]
+                except ValueError:
+                    if value is not None:
+                        raise
+
+                    return [0]
 
         # Convert element to index of the elements
-        if is_range_value:
-            slider_value = list(map(lambda v: index_(opt, v), value))
-            start, end = slider_value
-            if start > end:
-                slider_value = [end, start]
-        else:
-            # Simplify future logic by always making value a list
-            try:
-                slider_value = [index_(opt, value)]
-            except ValueError:
-                if value is not None:
-                    raise
-
-                slider_value = [0]
+        slider_value = as_index_list(value)
 
         slider_proto = SliderProto()
         slider_proto.label = label
@@ -138,8 +189,9 @@ class SelectSliderMixin:
         slider_proto.data_type = SliderProto.INT
         slider_proto.options[:] = [str(format_func(option)) for option in opt]
         slider_proto.form_id = current_form_id(self.dg)
+        slider_proto.disabled = disabled
         if help is not None:
-            slider_proto.help = help
+            slider_proto.help = dedent(help)
 
         def deserialize_select_slider(ui_value, widget_id=""):
             if not ui_value:
@@ -153,8 +205,7 @@ class SelectSliderMixin:
             return tuple(return_value) if is_range_value else return_value[0]
 
         def serialize_select_slider(v):
-            to_serialize = v if is_range_value else [v]
-            return [index_(opt, u) for u in to_serialize]
+            return as_index_list(v)
 
         current_value, set_frontend_value = register_widget(
             "slider",
@@ -165,6 +216,7 @@ class SelectSliderMixin:
             kwargs=kwargs,
             deserializer=deserialize_select_slider,
             serializer=serialize_select_slider,
+            ctx=ctx,
         )
 
         if set_frontend_value:
