@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,14 +13,21 @@
 # limitations under the License.
 
 from datetime import date, time, datetime, timedelta, timezone
+from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
+from streamlit.type_util import Key, to_key
 from typing import Any, List, cast, Optional
+from textwrap import dedent
 
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.js_number import JSNumber
 from streamlit.js_number import JSNumberBoundsException
 from streamlit.proto.Slider_pb2 import Slider as SliderProto
-from streamlit.proto.WidgetStates_pb2 import WidgetState as WidgetStateProto
+from streamlit.state.session_state import (
+    WidgetArgs,
+    WidgetCallback,
+    WidgetKwargs,
+)
 from streamlit.state.widgets import register_widget
 from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
@@ -29,17 +36,19 @@ from .utils import check_callback_rules, check_session_state_rules
 class SliderMixin:
     def slider(
         self,
-        label,
+        label: str,
         min_value=None,
         max_value=None,
         value=None,
         step=None,
         format=None,
-        key=None,
-        help=None,
-        on_change=None,
-        args=None,
-        kwargs=None,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
     ):
         """Display a slider widget.
 
@@ -82,8 +91,8 @@ class SliderMixin:
             Formatter for int/float supports: %d %e %f %g %i
             Formatter for date/time/datetime uses Moment.js notation:
             https://momentjs.com/docs/#/displaying/format/
-        key : str
-            An optional string to use as the unique key for the widget.
+        key : str or int
+            An optional string or integer to use as the unique key for the widget.
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
@@ -95,6 +104,9 @@ class SliderMixin:
             An optional tuple of args to pass to the callback.
         kwargs : dict
             An optional dict of kwargs to pass to the callback.
+        disabled : bool
+            An optional boolean, which disables the slider if set to True. The
+            default is False. This argument can only be supplied by keyword.
 
         Returns
         -------
@@ -131,7 +143,46 @@ class SliderMixin:
         ...     format="MM/DD/YY - hh:mm")
         >>> st.write("Start time:", start_time)
 
+        .. output::
+           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.slider.py
+           height: 300px
+
         """
+        ctx = get_script_run_ctx()
+        return self._slider(
+            label=label,
+            min_value=min_value,
+            max_value=max_value,
+            value=value,
+            step=step,
+            format=format,
+            key=key,
+            help=help,
+            on_change=on_change,
+            args=args,
+            kwargs=kwargs,
+            disabled=disabled,
+            ctx=ctx,
+        )
+
+    def _slider(
+        self,
+        label: str,
+        min_value=None,
+        max_value=None,
+        value=None,
+        step=None,
+        format=None,
+        key: Optional[Key] = None,
+        help: Optional[str] = None,
+        on_change: Optional[WidgetCallback] = None,
+        args: Optional[WidgetArgs] = None,
+        kwargs: Optional[WidgetKwargs] = None,
+        *,  # keyword-only arguments:
+        disabled: bool = False,
+        ctx: Optional[ScriptRunContext] = None,
+    ):
+        key = to_key(key)
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=value, key=key)
 
@@ -390,8 +441,9 @@ class SliderMixin:
         slider_proto.data_type = data_type
         slider_proto.options[:] = []
         slider_proto.form_id = current_form_id(self.dg)
+        slider_proto.disabled = disabled
         if help is not None:
-            slider_proto.help = help
+            slider_proto.help = dedent(help)
 
         def deserialize_slider(ui_value: Optional[List[float]], widget_id=""):
             if ui_value is not None:
@@ -415,7 +467,8 @@ class SliderMixin:
             return val[0] if single_value else tuple(val)
 
         def serialize_slider(v: Any) -> List[Any]:
-            value = [v] if single_value else list(v)
+            range_value = isinstance(v, (list, tuple))
+            value = list(v) if range_value else [v]
             if data_type == SliderProto.DATE:
                 value = [_datetime_to_micros(_date_to_datetime(v)) for v in value]
             if data_type == SliderProto.TIME:
@@ -433,6 +486,7 @@ class SliderMixin:
             kwargs=kwargs,
             deserializer=deserialize_slider,
             serializer=serialize_slider,
+            ctx=ctx,
         )
 
         if set_frontend_value:
