@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,32 +12,50 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import asyncio
 import logging
 import os
-import threading
 import socket
 import sys
 import errno
+import time
 import traceback
 import click
 from enum import Enum
-from typing import Any, Dict, Optional, TYPE_CHECKING
+from typing import (
+    Any,
+    Dict,
+    Optional,
+    Tuple,
+    Callable,
+    Awaitable,
+    Generator,
+    List,
+)
 
 import tornado.concurrent
 import tornado.gen
 import tornado.ioloop
+import tornado.locks
 import tornado.netutil
 import tornado.web
 import tornado.websocket
+from tornado.websocket import WebSocketHandler
+from tornado.httpserver import HTTPServer
+from tornado.ioloop import IOLoop
 
 from streamlit import config
 from streamlit import file_util
 from streamlit import util
+from streamlit.caching import get_memo_stats_provider, get_singleton_stats_provider
 from streamlit.config_option import ConfigOption
 from streamlit.forward_msg_cache import ForwardMsgCache
 from streamlit.forward_msg_cache import create_reference_msg
 from streamlit.forward_msg_cache import populate_hash_if_needed
-from streamlit.report_session import ReportSession
+from streamlit.in_memory_file_manager import in_memory_file_manager
+from streamlit.legacy_caching.caching import _mem_caches
+from streamlit.app_session import AppSession
+from streamlit.stats import StatsHandler, StatsManager
 from streamlit.uploaded_file_manager import UploadedFileManager
 from streamlit.logger import get_logger
 from streamlit.components.v1.components import ComponentRegistry
@@ -48,26 +66,29 @@ from streamlit.server.upload_file_request_handler import (
     UploadFileRequestHandler,
     UPLOAD_FILE_ROUTE,
 )
+
+from streamlit.session_data import SessionData
+from streamlit.state.session_state import (
+    SCRIPT_RUN_WITHOUT_ERRORS_KEY,
+    SessionStateStatProvider,
+)
 from streamlit.server.routes import AddSlashHandler
 from streamlit.server.routes import AssetsFileHandler
 from streamlit.server.routes import DebugHandler
 from streamlit.server.routes import HealthHandler
 from streamlit.server.routes import MediaFileHandler
 from streamlit.server.routes import MessageCacheHandler
-from streamlit.server.routes import MetricsHandler
 from streamlit.server.routes import StaticFileHandler
-from streamlit.server.server_util import MESSAGE_SIZE_LIMIT
 from streamlit.server.server_util import is_cacheable_msg
 from streamlit.server.server_util import is_url_from_allowed_origins
 from streamlit.server.server_util import make_url_path_regex
 from streamlit.server.server_util import serialize_forward_msg
+from streamlit.server.server_util import get_max_message_size_bytes
+from streamlit.watcher.local_sources_watcher import LocalSourcesWatcher
 
-if TYPE_CHECKING:
-    from streamlit.report import Report
 
 LOGGER = get_logger(__name__)
 
-
 TORNADO_SETTINGS = {
     # Gzip HTTP responses.
     "compress_response": True,
@@ -80,11 +101,8 @@ TORNADO_SETTINGS = {
     # If we don't get a ping response within 30s, the connection
     # is timed out.
     "websocket_ping_timeout": 30,
-    # Set the websocket message size. The default value is too low.
-    "websocket_max_message_size": MESSAGE_SIZE_LIMIT,
 }
 
-
 # When server.port is not available it will look for the next available port
 # up to MAX_PORT_SEARCH_RETRIES.
 MAX_PORT_SEARCH_RETRIES = 100
@@ -93,22 +111,25 @@ MAX_PORT_SEARCH_RETRIES = 100
 # to an unix socket.
 UNIX_SOCKET_PREFIX = "unix://"
 
+# Wait for the script run result for 60s and if no result is available give up
+SCRIPT_RUN_CHECK_TIMEOUT = 60
 
-class SessionInfo(object):
+
+class SessionInfo:
     """Type stored in our _session_info_by_id dict.
 
-    For each ReportSession, the server tracks that session's
+    For each AppSession, the server tracks that session's
     report_run_count. This is used to track the age of messages in
     the ForwardMsgCache.
     """
 
-    def __init__(self, ws, session: ReportSession):
+    def __init__(self, ws: WebSocketHandler, session: AppSession):
         """Initialize a SessionInfo instance.
 
         Parameters
         ----------
-        session : ReportSession
-            The ReportSession object.
+        session : AppSession
+            The AppSession object.
         ws : _BrowserWebSocketHandler
             The websocket that owns this report.
         """
@@ -133,16 +154,16 @@ class RetriesExceeded(Exception):
     pass
 
 
-def server_port_is_manually_set():
+def server_port_is_manually_set() -> bool:
     return config.is_manually_set("server.port")
 
 
-def server_address_is_unix_socket():
+def server_address_is_unix_socket() -> bool:
     address = config.get_option("server.address")
-    return address and address.startswith(UNIX_SOCKET_PREFIX)
+    return address is not None and address.startswith(UNIX_SOCKET_PREFIX)
 
 
-def start_listening(app):
+def start_listening(app: tornado.web.Application) -> None:
     """Makes the server start listening at the configured port.
 
     In case the port is already taken it tries listening to the next available
@@ -150,7 +171,7 @@ def start_listening(app):
 
     """
 
-    http_server = tornado.httpserver.HTTPServer(
+    http_server = HTTPServer(
         app, max_buffer_size=config.get_option("server.maxUploadSize") * 1024 * 1024
     )
 
@@ -160,7 +181,7 @@ def start_listening(app):
         start_listening_tcp_socket(http_server)
 
 
-def start_listening_unix_socket(http_server):
+def start_listening_unix_socket(http_server: HTTPServer) -> None:
     address = config.get_option("server.address")
     file_name = os.path.expanduser(address[len(UNIX_SOCKET_PREFIX) :])
 
@@ -168,9 +189,10 @@ def start_listening_unix_socket(http_server):
     http_server.add_socket(unix_socket)
 
 
-def start_listening_tcp_socket(http_server):
+def start_listening_tcp_socket(http_server: HTTPServer) -> None:
     call_count = 0
 
+    port = None
     while call_count < MAX_PORT_SEARCH_RETRIES:
         address = config.get_option("server.address")
         port = config.get_option("server.port")
@@ -203,33 +225,28 @@ def start_listening_tcp_socket(http_server):
 
     if call_count >= MAX_PORT_SEARCH_RETRIES:
         raise RetriesExceeded(
-            "Cannot start Streamlit server. Port %s is already in use, and "
-            "Streamlit was unable to find a free port after %s attempts.",
-            port,
-            MAX_PORT_SEARCH_RETRIES,
+            f"Cannot start Streamlit server. Port {port} is already in use, and "
+            f"Streamlit was unable to find a free port after {MAX_PORT_SEARCH_RETRIES} attempts.",
         )
 
 
-class Server(object):
-
-    _singleton = None  # type: Optional[Server]
+class Server:
+    _singleton: Optional["Server"] = None
 
     @classmethod
-    def get_current(cls):
+    def get_current(cls) -> "Server":
         """
         Returns
         -------
         Server
             The singleton Server object.
         """
-        if cls._singleton is None:
+        if Server._singleton is None:
             raise RuntimeError("Server has not been initialized yet")
 
         return Server._singleton
 
-    def __init__(
-        self, ioloop: tornado.ioloop.IOLoop, script_path: str, command_line: str
-    ):
+    def __init__(self, ioloop: IOLoop, script_path: str, command_line: Optional[str]):
         """Create the server. It won't be started yet."""
         if Server._singleton is not None:
             raise RuntimeError("Server already initialized. Use .get_current() instead")
@@ -240,19 +257,31 @@ class Server(object):
 
         self._ioloop = ioloop
         self._script_path = script_path
-        self._command_line = command_line
+        self._command_line = command_line if command_line is not None else ""
 
-        # Mapping of ReportSession.id -> SessionInfo.
+        # Mapping of AppSession.id -> SessionInfo.
         self._session_info_by_id: Dict[str, SessionInfo] = {}
 
-        self._must_stop = threading.Event()
-        self._state = None
-        self._set_state(State.INITIAL)
+        self._must_stop = tornado.locks.Event()
+        self._state = State.INITIAL
         self._message_cache = ForwardMsgCache()
         self._uploaded_file_mgr = UploadedFileManager()
         self._uploaded_file_mgr.on_files_updated.connect(self.on_files_updated)
-        self._report = None  # type: Optional[Report]
-        self._preheated_session_id = None  # type: Optional[str]
+        self._session_data: Optional[SessionData] = None
+        self._has_connection = tornado.locks.Condition()
+        self._need_send_data = tornado.locks.Event()
+
+        # StatsManager
+        self._stats_mgr = StatsManager()
+        self._stats_mgr.register_provider(get_memo_stats_provider())
+        self._stats_mgr.register_provider(get_singleton_stats_provider())
+        self._stats_mgr.register_provider(_mem_caches)
+        self._stats_mgr.register_provider(self._message_cache)
+        self._stats_mgr.register_provider(in_memory_file_manager)
+        self._stats_mgr.register_provider(self._uploaded_file_mgr)
+        self._stats_mgr.register_provider(
+            SessionStateStatProvider(self._session_info_by_id)
+        )
 
     def __repr__(self) -> str:
         return util.repr_(self)
@@ -261,8 +290,8 @@ class Server(object):
     def script_path(self) -> str:
         return self._script_path
 
-    def get_session_by_id(self, session_id: str) -> Optional[ReportSession]:
-        """Return the ReportSession corresponding to the given id, or None if
+    def get_session_by_id(self, session_id: str) -> Optional[AppSession]:
+        """Return the AppSession corresponding to the given id, or None if
         no such session exists."""
         session_info = self._get_session_info(session_id)
         if session_info is None:
@@ -287,7 +316,7 @@ class Server(object):
         """
         return self._session_info_by_id.get(session_id, None)
 
-    def start(self, on_started):
+    def start(self, on_started: Callable[["Server"], Any]) -> None:
         """Start the server.
 
         Parameters
@@ -311,21 +340,11 @@ class Server(object):
 
         self._ioloop.spawn_callback(self._loop_coroutine, on_started)
 
-    def get_debug(self) -> Dict[str, Dict[str, Any]]:
-        if self._report:
-            return {"report": self._report.get_debug()}
-        return {}
-
-    def _create_app(self):
-        """Create our tornado web app.
-
-        Returns
-        -------
-        tornado.web.Application
-
-        """
+    def _create_app(self) -> tornado.web.Application:
+        """Create our tornado web app."""
         base = config.get_option("server.baseUrlPath")
-        routes = [
+
+        routes: List[Any] = [
             (
                 make_url_path_regex(base, "stream"),
                 _BrowserWebSocketHandler,
@@ -337,12 +356,16 @@ class Server(object):
                 dict(callback=lambda: self.is_ready_for_browser_connection),
             ),
             (make_url_path_regex(base, "debugz"), DebugHandler, dict(server=self)),
-            (make_url_path_regex(base, "metrics"), MetricsHandler),
             (
                 make_url_path_regex(base, "message"),
                 MessageCacheHandler,
                 dict(cache=self._message_cache),
             ),
+            (
+                make_url_path_regex(base, "st-metrics"),
+                StatsHandler,
+                dict(stats_manager=self._stats_mgr),
+            ),
             (
                 make_url_path_regex(
                     base,
@@ -367,6 +390,17 @@ class Server(object):
             ),
         ]
 
+        if config.get_option("server.scriptHealthCheckEnabled"):
+            routes.extend(
+                [
+                    (
+                        make_url_path_regex(base, "script-health-check"),
+                        HealthHandler,
+                        dict(callback=lambda: self.does_script_run_without_error()),
+                    )
+                ]
+            )
+
         if config.get_option("global.developmentMode"):
             LOGGER.debug("Serving static content from the Node dev server")
         else:
@@ -385,32 +419,77 @@ class Server(object):
             )
 
         return tornado.web.Application(
-            routes,  # type: ignore[arg-type]
+            routes,
             cookie_secret=config.get_option("server.cookieSecret"),
             xsrf_cookies=config.get_option("server.enableXsrfProtection"),
+            # Set the websocket message size. The default value is too low.
+            websocket_max_message_size=get_max_message_size_bytes(),
             **TORNADO_SETTINGS,  # type: ignore[arg-type]
         )
 
-    def _set_state(self, new_state):
+    def _set_state(self, new_state: State) -> None:
         LOGGER.debug("Server state: %s -> %s" % (self._state, new_state))
         self._state = new_state
 
     @property
-    def is_ready_for_browser_connection(self):
-        return self._state not in (State.INITIAL, State.STOPPING, State.STOPPED)
+    async def is_ready_for_browser_connection(self) -> Tuple[bool, str]:
+        if self._state not in (State.INITIAL, State.STOPPING, State.STOPPED):
+            return True, "ok"
+
+        return False, "unavailable"
+
+    async def does_script_run_without_error(self) -> Tuple[bool, str]:
+        """Load and execute the app's script to verify it runs without an error.
+
+        Returns
+        -------
+        (True, "ok") if the script completes without error, or (False, err_msg)
+        if the script raises an exception.
+        """
+        session_data = SessionData(self._script_path, self._command_line)
+        local_sources_watcher = LocalSourcesWatcher(session_data)
+        session = AppSession(
+            ioloop=self._ioloop,
+            session_data=session_data,
+            uploaded_file_manager=self._uploaded_file_mgr,
+            message_enqueued_callback=self._enqueued_some_message,
+            local_sources_watcher=local_sources_watcher,
+        )
+
+        try:
+            session.request_rerun(None)
+
+            now = time.perf_counter()
+            while (
+                SCRIPT_RUN_WITHOUT_ERRORS_KEY not in session.session_state
+                and (time.perf_counter() - now) < SCRIPT_RUN_CHECK_TIMEOUT
+            ):
+                await tornado.gen.sleep(0.1)
+
+            if SCRIPT_RUN_WITHOUT_ERRORS_KEY not in session.session_state:
+                return False, "timeout"
+
+            ok = session.session_state[SCRIPT_RUN_WITHOUT_ERRORS_KEY]
+            msg = "ok" if ok else "error"
+
+            return ok, msg
+        finally:
+            session.shutdown()
 
     @property
-    def browser_is_connected(self):
+    def browser_is_connected(self) -> bool:
         return self._state == State.ONE_OR_MORE_BROWSERS_CONNECTED
 
     @property
-    def is_running_hello(self):
+    def is_running_hello(self) -> bool:
         from streamlit.hello import hello
 
         return self._script_path == hello.__file__
 
     @tornado.gen.coroutine
-    def _loop_coroutine(self, on_started=None):
+    def _loop_coroutine(
+        self, on_started: Optional[Callable[["Server"], Any]] = None
+    ) -> Generator[Any, None, None]:
         try:
             if self._state == State.INITIAL:
                 self._set_state(State.WAITING_FOR_FIRST_BROWSER)
@@ -425,9 +504,15 @@ class Server(object):
             while not self._must_stop.is_set():
 
                 if self._state == State.WAITING_FOR_FIRST_BROWSER:
-                    pass
+                    yield tornado.gen.convert_yielded(
+                        asyncio.wait(
+                            [self._must_stop.wait(), self._has_connection.wait()],
+                            return_when=asyncio.FIRST_COMPLETED,
+                        )
+                    )
 
                 elif self._state == State.ONE_OR_MORE_BROWSERS_CONNECTED:
+                    self._need_send_data.clear()
 
                     # Shallow-clone our sessions into a list, so we can iterate
                     # over it and not worry about whether it's being changed
@@ -435,34 +520,42 @@ class Server(object):
                     session_infos = list(self._session_info_by_id.values())
 
                     for session_info in session_infos:
-                        if session_info.ws is None:
-                            # Preheated.
-                            continue
                         msg_list = session_info.session.flush_browser_queue()
                         for msg in msg_list:
                             try:
                                 self._send_message(session_info, msg)
                             except tornado.websocket.WebSocketClosedError:
-                                self._close_report_session(session_info.session.id)
+                                self._close_app_session(session_info.session.id)
                             yield
                         yield
+                    yield tornado.gen.sleep(0.01)
 
                 elif self._state == State.NO_BROWSERS_CONNECTED:
-                    pass
+                    yield tornado.gen.convert_yielded(
+                        asyncio.wait(
+                            [self._must_stop.wait(), self._has_connection.wait()],
+                            return_when=asyncio.FIRST_COMPLETED,
+                        )
+                    )
 
                 else:
                     # Break out of the thread loop if we encounter any other state.
                     break
 
-                yield tornado.gen.sleep(0.01)
+                yield tornado.gen.convert_yielded(
+                    asyncio.wait(
+                        [self._must_stop.wait(), self._need_send_data.wait()],
+                        return_when=asyncio.FIRST_COMPLETED,
+                    )
+                )
 
-            # Shut down all ReportSessions
+            # Shut down all AppSessions
             for session_info in list(self._session_info_by_id.values()):
                 session_info.session.shutdown()
 
             self._set_state(State.STOPPED)
 
-        except Exception as e:
+        except Exception:
             # Can't just re-raise here because co-routines use Tornado
             # exceptions for control flow, which appears to swallow the reraised
             # exception.
@@ -476,7 +569,7 @@ Please report this bug at https://github.com/streamlit/streamlit/issues.
         finally:
             self._on_stopped()
 
-    def _send_message(self, session_info, msg):
+    def _send_message(self, session_info: SessionInfo, msg: ForwardMsg) -> None:
         """Send a message to a client.
 
         If the client is likely to have already cached the message, we may
@@ -499,7 +592,6 @@ Please report this bug at https://github.com/streamlit/streamlit/issues.
             if self._message_cache.has_message_reference(
                 msg, session_info.session, session_info.report_run_count
             ):
-
                 # This session has probably cached this message. Send
                 # a reference instead.
                 LOGGER.debug("Sending cached message ref (hash=%s)" % msg.hash)
@@ -513,11 +605,11 @@ Please report this bug at https://github.com/streamlit/streamlit/issues.
                 msg, session_info.session, session_info.report_run_count
             )
 
-        # If this was a `report_finished` message, we increment the
+        # If this was a `script_finished` message, we increment the
         # report_run_count for this session, and update the cache
         if (
-            msg.WhichOneof("type") == "report_finished"
-            and msg.report_finished == ForwardMsg.FINISHED_SUCCESSFULLY
+            msg.WhichOneof("type") == "script_finished"
+            and msg.script_finished == ForwardMsg.FINISHED_SUCCESSFULLY
         ):
             LOGGER.debug(
                 "Report finished successfully; "
@@ -533,12 +625,18 @@ Please report this bug at https://github.com/streamlit/streamlit/issues.
         # Ship it off!
         session_info.ws.write_message(serialize_forward_msg(msg_to_send), binary=True)
 
-    def stop(self):
+    def _enqueued_some_message(self) -> None:
+        self._ioloop.add_callback(self._need_send_data.set)
+
+    def stop(self, from_signal=False) -> None:
         click.secho("  Stopping...", fg="blue")
         self._set_state(State.STOPPING)
-        self._must_stop.set()
+        if from_signal:
+            self._ioloop.add_callback_from_signal(self._must_stop.set)
+        else:
+            self._ioloop.add_callback(self._must_stop.set)
 
-    def _on_stopped(self):
+    def _on_stopped(self) -> None:
         """Called when our runloop is exiting, to shut down the ioloop.
         This will end our process.
 
@@ -547,72 +645,46 @@ Please report this bug at https://github.com/streamlit/streamlit/issues.
         """
         self._ioloop.stop()
 
-    def add_preheated_report_session(self):
-        """Register a fake browser with the server and run the script.
-
-        This is used to start running the user's script even before the first
-        browser connects.
-        """
-        session = self._create_or_reuse_report_session(ws=None)
-        session.handle_rerun_script_request(is_preheat=True)
-
-    def _create_or_reuse_report_session(self, ws):
+    def _create_app_session(self, ws: WebSocketHandler) -> AppSession:
         """Register a connected browser with the server.
 
         Parameters
         ----------
-        ws : _BrowserWebSocketHandler or None
-            The newly-connected websocket handler or None if preheated
-            connection.
+        ws : _BrowserWebSocketHandler
+            The newly-connected websocket handler.
 
         Returns
         -------
-        ReportSession
-            The newly-created ReportSession for this browser connection.
+        AppSession
+            The newly-created AppSession for this browser connection.
 
         """
-        if self._preheated_session_id is not None:
-            assert len(self._session_info_by_id) == 1
-            assert ws is not None
+        session_data = SessionData(self._script_path, self._command_line)
+        local_sources_watcher = LocalSourcesWatcher(session_data)
+        session = AppSession(
+            ioloop=self._ioloop,
+            session_data=session_data,
+            uploaded_file_manager=self._uploaded_file_mgr,
+            message_enqueued_callback=self._enqueued_some_message,
+            local_sources_watcher=local_sources_watcher,
+        )
 
-            session_id = self._preheated_session_id
-            self._preheated_session_id = None
+        LOGGER.debug(
+            "Created new session for ws %s. Session ID: %s", id(ws), session.id
+        )
 
-            session_info = self._session_info_by_id[session_id]
-            session_info.ws = ws
-            session = session_info.session
-
-            LOGGER.debug(
-                "Reused preheated session for ws %s. Session ID: %s", id(ws), session_id
-            )
-
-        else:
-            session = ReportSession(
-                ioloop=self._ioloop,
-                script_path=self._script_path,
-                command_line=self._command_line,
-                uploaded_file_manager=self._uploaded_file_mgr,
-            )
-
-            LOGGER.debug(
-                "Created new session for ws %s. Session ID: %s", id(ws), session.id
-            )
-
-            assert session.id not in self._session_info_by_id, (
-                "session.id '%s' registered multiple times!" % session.id
-            )
+        assert (
+            session.id not in self._session_info_by_id
+        ), f"session.id '{session.id}' registered multiple times!"
 
         self._session_info_by_id[session.id] = SessionInfo(ws, session)
-
-        if ws is None:
-            self._preheated_session_id = session.id
-        else:
-            self._set_state(State.ONE_OR_MORE_BROWSERS_CONNECTED)
+        self._set_state(State.ONE_OR_MORE_BROWSERS_CONNECTED)
+        self._has_connection.notify_all()
 
         return session
 
-    def _close_report_session(self, session_id):
-        """Shutdown and remove a ReportSession.
+    def _close_app_session(self, session_id: str) -> None:
+        """Shutdown and remove a AppSession.
 
         This function may be called multiple times for the same session,
         which is not an error. (Subsequent calls just no-op.)
@@ -620,7 +692,7 @@ Please report this bug at https://github.com/streamlit/streamlit/issues.
         Parameters
         ----------
         session_id : str
-            The ReportSession's id string.
+            The AppSession's id string.
         """
         if session_id in self._session_info_by_id:
             session_info = self._session_info_by_id[session_id]
@@ -631,7 +703,7 @@ Please report this bug at https://github.com/streamlit/streamlit/issues.
             self._set_state(State.NO_BROWSERS_CONNECTED)
 
 
-class _BrowserWebSocketHandler(tornado.websocket.WebSocketHandler):
+class _BrowserWebSocketHandler(WebSocketHandler):
     """Handles a WebSocket connection from the browser"""
 
     def initialize(self, server):
@@ -643,22 +715,23 @@ class _BrowserWebSocketHandler(tornado.websocket.WebSocketHandler):
         # See https://www.tornadoweb.org/en/stable/guide/security.html#cross-site-request-forgery-protection
         # for more details.
         if config.get_option("server.enableXsrfProtection"):
-            self.xsrf_token
+            _ = self.xsrf_token
 
-    def check_origin(self, origin):
+    def check_origin(self, origin: str) -> bool:
         """Set up CORS."""
         return super().check_origin(origin) or is_url_from_allowed_origins(origin)
 
-    def open(self):
-        self._session = self._server._create_or_reuse_report_session(self)
+    def open(self, *args, **kwargs) -> Optional[Awaitable[None]]:
+        self._session = self._server._create_app_session(self)
+        return None
 
-    def on_close(self):
+    def on_close(self) -> None:
         if not self._session:
             return
-        self._server._close_report_session(self._session.id)
+        self._server._close_app_session(self._session.id)
         self._session = None
 
-    def get_compression_options(self):
+    def get_compression_options(self) -> Optional[Dict[Any, Any]]:
         """Enable WebSocket compression.
 
         Returning an empty dict enables websocket compression. Returning
@@ -671,7 +744,7 @@ class _BrowserWebSocketHandler(tornado.websocket.WebSocketHandler):
         return None
 
     @tornado.gen.coroutine
-    def on_message(self, payload):
+    def on_message(self, payload: bytes) -> None:
         if not self._session:
             return
 
@@ -683,9 +756,7 @@ class _BrowserWebSocketHandler(tornado.websocket.WebSocketHandler):
 
             LOGGER.debug("Received the following back message:\n%s", msg)
 
-            if msg_type == "cloud_upload":
-                yield self._session.handle_save_request(self)
-            elif msg_type == "rerun_script":
+            if msg_type == "rerun_script":
                 self._session.handle_rerun_script_request(msg.rerun_script)
             elif msg_type == "load_git_info":
                 self._session.handle_git_information_request()
@@ -693,7 +764,7 @@ class _BrowserWebSocketHandler(tornado.websocket.WebSocketHandler):
                 self._session.handle_clear_cache_request()
             elif msg_type == "set_run_on_save":
                 self._session.handle_set_run_on_save_request(msg.set_run_on_save)
-            elif msg_type == "stop_report":
+            elif msg_type == "stop_script":
                 self._session.handle_stop_script_request()
             elif msg_type == "close_connection":
                 if config.get_option("global.developmentMode"):
@@ -711,7 +782,7 @@ class _BrowserWebSocketHandler(tornado.websocket.WebSocketHandler):
             self._session.enqueue_exception(e)
 
 
-def _set_tornado_log_levels():
+def _set_tornado_log_levels() -> None:
     if not config.get_option("global.developmentMode"):
         # Hide logs unless they're super important.
         # Example of stuff we don't care about: 404 about .js.map files.
